# rpkg bash completion

_rpkg()
{
    COMPREPLY=()

    in_array()
    {
        local i
        for i in $2; do
            [[ $i = $1 ]] && return 0
        done
        return 1
    }

    _filedir_exclude_paths()
    {
        _filedir "$@"
        for ((i=0; i<=${#COMPREPLY[@]}; i++)); do
            [[ ${COMPREPLY[$i]} =~ /?\.git/? ]] && unset COMPREPLY[$i]
        done
    }

    local cur prev
    # _get_comp_words_by_ref is in bash-completion >= 1.2, which EL-5 lacks.
    if type _get_comp_words_by_ref &>/dev/null; then
        _get_comp_words_by_ref cur prev
    else
        cur="${COMP_WORDS[COMP_CWORD]}"
        prev="${COMP_WORDS[COMP_CWORD-1]}"
    fi

    # global options

    local options="-h --help -v --verbose -q --quiet"
    local options_value="-u --user -p --path --config -c --config-file -C"
    local commands="add clean clog clone config commit ci compile copr-build build diff gimmespec import install lint local \
        log merge new nvr prep pull push reset sources spec srpm status switch tag unused-patches upload verify-files"

    # parse main options and get command

    local command=
    local command_first=
    local path=

    local i w
    for (( i = 0; i < ${#COMP_WORDS[*]} - 1; i++ )); do
        w="${COMP_WORDS[$i]}"
        # option
        if [[ ${w:0:1} = - ]]; then
            if in_array "$w" "$options_value"; then
                ((i++))
                [[ "$w" = --path ]] && path="${COMP_WORDS[$i]}"
            fi
        # command
        elif in_array "$w" "$commands"; then
            command=$w
            command_first=$((i+1))
            break
        fi
    done

    # complete base options

    if [[ -z $command ]]; then
        if [[ $cur == -* ]]; then
            COMPREPLY=( $(compgen -W "$options $options_value" -- "$cur") )
            return 0
        fi

        case "$prev" in
            --config-file|-C)
                _filedir_exclude_paths
                ;;
            --user|-u)
                ;;
            --path|-p)
                _filedir_exclude_paths
                ;;
            *)
                COMPREPLY=( $(compgen -W "$commands" -- "$cur") )
                ;;
        esac

        return 0
    fi

    # parse command specific options

    local options=
    local options_arches= options_branch= options_tag= options_string= options_bool= options_file= options_dir= options_srpm= options_spec=
    local after= after_more=

    case $command in
        help)
        ;;
        add)
            options="--all -A"
            after="file"
            after_more=true
        ;;
        clean)
            options="--dry-run -n -x --force -f -d"
        ;;
        clog)
            options="--raw"
        ;;
        clone)
            options="--anonymous -a --branches -B"
            options_branch="--branch -b"
        ;;
        config)
            options=""
        ;;
        ci|commit)
            options="--message -m --all -a --amend --with-changelog --tag -t --push -p --signoff -s"
            options_file="--file -F"
            after="file"
            after_more=true
        ;;
        compile)
            options="--target --with --without --short-circuit --nocheck"
            options_dir="--outdir"
            options_spec="--spec"
        ;;
        copr-build|build)
            options="--wait -w"
            options_dir="--outdir"
            options_file="--config"
            options_spec="--spec"
        ;;
        diff)
            options="--cached"
            after="file"
            after_more=true
        ;;
        gimmespec)
            options=""
        ;;
        import)
            options="--repo-path --skip-diffs --offline"
            after="srpm"
        ;;
        install)
            options="--target --with --without --short-circuit --nocheck"
            options_dir="--outdir"
            options_spec="--spec"
        ;;
        lint)
            options="--info -i"
            options_file="--rpmlint-config -r"
            after="file"
        ;;
        local)
            options="--target --with --without --nocheck"
            options_dir="--outdir"
            options_spec="--spec"
        ;;
        log)
            options="--graph --all --show-signature"
            # TODO: after: revision, path
        ;;
        merge)
            options="--no-commit --squash --message -m --file -F --signoff --abort --continue"
            # TODO: after: commits
        ;;
        new)
            options_dir="--outdir"
            options_spec="--spec"
        ;;
        nvr)
            options_dir="--outdir"
            options_spec="--spec"
        ;;
        prep)
            options="--target --with --without"
            options_dir="--outdir"
            options_spec="--spec"
        ;;
        pull)
            options="--rebase --no-rebase"
        ;;
        push)
            options="--follow-tags --force -f"
        ;;
        reset)
            options="--soft --mixed --hard"
            after="file"
            # TODO: after: commit|file instead of just file
        ;;
        sources)
            options="--repo-path"
            options_dir="--outdir"
        ;;
        spec)
            options="--sources --print -p"
            options_dir="--outdir"
            options_spec="--spec"
        ;;
        srpm)
            options="--target --with --without"
            options_dir="--outdir"
            options_spec="--spec"
        ;;
        status)
            after="file"
            after_more=true
        ;;
        switch)
            options="--create -c --detach -d --list-branches -l --fetch"
            after="branch"
        ;;
        tag)
            options="--list -l --delete -d --skip-edit -s --name -n --version -v --release -r"
            after="tag"
        ;;
        unused-patches)
            options=""
        ;;
        upload)
            options="--repo-path --offline"
            after="file"
            after_more=true
        ;;
        verify-files)
            after="file"
        ;;
    esac

    local all_options="--help $options"
    local all_options_value="$options_target $options_arches $options_branch $options_string $options_file $options_dir $options_srpm $options_spec $options_bool"

    # count non-option parameters

    local i w
    local last_option=
    local after_counter=0
    for (( i = $command_first; i < ${#COMP_WORDS[*]} - 1; i++)); do
        w="${COMP_WORDS[$i]}"
        if [[ ${w:0:1} = - ]]; then
            if in_array "$w" "$all_options"; then
                last_option="$w"
                continue
            elif in_array "$w" "$all_options_value"; then
                last_option="$w"
                ((i++))
                continue
            fi
        fi
        in_array "$last_option" "$options_arches" || ((after_counter++))
    done

    # completion

    if [[ -n $options_target ]] && in_array "$prev" "$options_target"; then
        COMPREPLY=( $(compgen -W "$(_rpkg_target)" -- "$cur") )

    elif [[ -n $options_arches ]] && in_array "$last_option" "$options_arches"; then
        COMPREPLY=( $(compgen -W "$(_rpkg_arch) $all_options" -- "$cur") )

    elif [[ -n $options_spec ]] && in_array "$prev" "$options_spec"; then
        _filedir_exclude_paths "@(spec|spec.rpkg)"

    elif [[ -n $options_srpm ]] && in_array "$prev" "$options_srpm"; then
        _filedir_exclude_paths "@(src.rpm)"

    elif [[ -n $options_branch ]] && in_array "$prev" "$options_branch"; then
        COMPREPLY=( $(compgen -W "$(_rpkg_branch "$path")" -- "$cur") )

    elif [[ -n $options_tag ]] && in_array "$prev" "$options_tag"; then
        COMPREPLY=( $(compgen -W "$(_rpkg_tag)" -- "$cur") )

    elif [[ -n $options_file ]] && in_array "$prev" "$options_file"; then
        _filedir_exclude_paths

    elif [[ -n $options_dir ]] && in_array "$prev" "$options_dir"; then
        _filedir_exclude_paths -d

    elif [[ -n $options_bool ]] && in_array "$prev" "$options_bool"; then
        COMPREPLY=( $(compgen -W "true false" -- "$cur") )

    elif [[ -n $options_string ]] && in_array "$prev" "$options_string"; then
        COMPREPLY=( )

    else
        local after_options=

        if [[ $after_counter -eq 0 ]] || [[ $after_more = true ]]; then
            case $after in
                file)    _filedir_exclude_paths ;;
                spec)    _filedir_exclude_paths "@(spec|spec.rpkg)" ;;
                srpm)    _filedir_exclude_paths "@(src.rpm)" ;;
                branch)  after_options="$(_rpkg_branch "$path")" ;;
                tag)     after_options="$(_rpkg_tag)";;
            esac
        fi

        if [[ $cur != -* ]]; then
            all_options=
            all_options_value=
        fi

        COMPREPLY+=( $(compgen -W "$all_options $all_options_value $after_options" -- "$cur" ) )
    fi

    return 0
} &&
complete -F _rpkg rpkg

_rpkg_target()
{
    return
}

_rpkg_arch()
{
    echo "i386 x86_64 ppc ppc64 s390 s390x sparc sparc64"
}

_rpkg_branch()
{
    local git_options= format="--format %(refname:short)"
    [[ -n $1 ]] && git_options="--git-dir=$1/.git"
    git $git_options for-each-ref $format 'refs/remotes' | sed 's,.*/,,'
    git $git_options for-each-ref $format 'refs/heads'
}

_rpkg_tag()
{
    git for-each-ref refs/tags/ --format='%(tag)'
}

# Local variables:
# mode: shell-script
# sh-basic-offset: 4
# sh-indent-comment: t
# indent-tabs-mode: nil
# End:
# ex: ts=4 sw=4 et filetype=sh
